'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.mapValuesAsync = mapValuesAsync;
exports.resolver = resolver;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function mapValuesAsync(obj, map) {
  if (obj !== Object(obj)) {
    throw new Error(obj + ' is not an object');
  }
  return Promise.all(Object.keys(obj).map(function (key) {
    return Promise.resolve(map(obj[key], key, obj)).then(function (value) {
      return {
        value: value,
        key: key
      };
    });
  })).then(function (kvps) {
    return kvps.reduce(function (memo, kvp) {
      return _extends({}, memo, _defineProperty({}, kvp.key, kvp.value));
    }, {});
  });
}

function isObject(item) {
  return (typeof item === 'undefined' ? 'undefined' : _typeof(item)) === 'object' && !Array.isArray(item) && item !== null;
}

var DEFAULT_NEXT_STATUS = function DEFAULT_NEXT_STATUS(_ref, newPath) {
  var depth = _ref.depth,
      path = _ref.path,
      status = _objectWithoutProperties(_ref, ['depth', 'path']);

  return _extends({}, status, {
    depth: depth + 1,
    path: [].concat(_toConsumableArray(path), [newPath])
  });
};

var DEFAULT_SHOULD_RESOLVE = function DEFAULT_SHOULD_RESOLVE() {
  return true;
};
var noop = function noop() {};

function resolver(context) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref2$nextStatus = _ref2.nextStatus,
      nextStatus = _ref2$nextStatus === undefined ? DEFAULT_NEXT_STATUS : _ref2$nextStatus,
      _ref2$maxDepth = _ref2.maxDepth,
      maxDepth = _ref2$maxDepth === undefined ? -1 : _ref2$maxDepth,
      _ref2$shouldResolve = _ref2.shouldResolve,
      shouldResolve = _ref2$shouldResolve === undefined ? DEFAULT_SHOULD_RESOLVE : _ref2$shouldResolve,
      _ref2$onDidResolve = _ref2.onDidResolve,
      onDidResolve = _ref2$onDidResolve === undefined ? noop : _ref2$onDidResolve,
      _ref2$onWillResolve = _ref2.onWillResolve,
      onWillResolve = _ref2$onWillResolve === undefined ? noop : _ref2$onWillResolve;

  var waitToResolve = function waitToResolve() {
    return Promise.resolve();
  };
  var didResolve = function didResolve() {
    onDidResolve.apply(undefined, arguments);
  };
  var willResolve = function willResolve() {
    onWillResolve.apply(undefined, arguments);
  };
  return function resolve(model) {
    var status = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { depth: 0, path: [] };

    if (status.depth > maxDepth && maxDepth >= 0) {
      didResolve(model, context, status);
      return Promise.resolve(model);
    }
    return waitToResolve(model, context, status).then(function () {
      if (model instanceof Function) {
        willResolve(model, context, status);
        if (!shouldResolve(context, status)) {
          didResolve(model, context, status);
          return Promise.resolve(model);
        }
        return resolve(model(context, status), status);
      }
      if (Array.isArray(model)) {
        return Promise.all(model.map(function (item, index, arr) {
          return resolve(item, nextStatus(status, index, arr));
        }));
      }
      if (model instanceof Promise) {
        return model.then(function (result) {
          return resolve(result, status);
        });
      }
      if (!isObject(model)) {
        didResolve(model, context, status);
        return Promise.resolve(model);
      }
      return mapValuesAsync(model, function (value, key, arr) {
        return resolve(value, nextStatus(status, key, arr));
      });
    });
  };
}

exports.default = resolver();